# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-09-26 21:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('fobi', '0006_auto_20160911_1549'),
    ]

    operations = [
        migrations.CreateModel(
            name='FormWizardFormEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('position', models.PositiveIntegerField(blank=True, null=True, verbose_name='Position')),
            ],
            options={
                'ordering': ['position'],
                'abstract': False,
                'verbose_name': 'Form wizard form entry',
                'verbose_name_plural': 'Form wizard form entries',
            },
        ),
        migrations.RemoveField(
            model_name='formentry',
            name='form_wizard_entry',
        ),
        migrations.RemoveField(
            model_name='formentry',
            name='position',
        ),
        migrations.AddField(
            model_name='formwizardentry',
            name='created',
            field=models.DateTimeField(auto_now_add=True, null=True, verbose_name='Created'),
        ),
        migrations.AddField(
            model_name='formwizardentry',
            name='success_page_message',
            field=models.TextField(blank=True, help_text='Custom message text to display after valid form is submitted', null=True, verbose_name='Success page body'),
        ),
        migrations.AddField(
            model_name='formwizardentry',
            name='success_page_title',
            field=models.CharField(blank=True, help_text='Custom message title to display after valid form is submitted', max_length=255, null=True, verbose_name='Success page title'),
        ),
        migrations.AddField(
            model_name='formwizardentry',
            name='updated',
            field=models.DateTimeField(auto_now=True, null=True, verbose_name='Updated'),
        ),
        migrations.AddField(
            model_name='formwizardformentry',
            name='form_entry',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='fobi.FormEntry', verbose_name='Form entry'),
        ),
        migrations.AddField(
            model_name='formwizardformentry',
            name='form_wizard_entry',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='fobi.FormWizardEntry', verbose_name='Form wizard entry'),
        ),
        migrations.AlterUniqueTogether(
            name='formwizardformentry',
            unique_together=set([('form_wizard_entry', 'form_entry')]),
        ),
    ]



from django.forms.widgets import RadioSelect
from django.utils.translation import ugettext_lazy as _

# Safe import of ``NumberInput``
try:
    from django.forms.widgets import NumberInput
except ImportError:
    from django.forms.widgets import TextInput

    class NumberInput(TextInput):
        """Number input."""

        input_type = 'number'

__title__ = 'fobi.widgets'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'NumberInput',
    'BooleanRadioSelect',
)


BOOLEAN_CHOICES = (
    (True, _("Yes")),
    (False, _("No"))
)


class BooleanRadioSelect(RadioSelect):
    """Boolean radio select for Django.

    :example:

    >>> class DummyForm(forms.Form):
    >>>     agree = forms.BooleanField(label=_("Agree?"),
    >>>                                required=False,
    >>>                                widget=BooleanRadioSelect)
    """

    def __init__(self, *args, **kwargs):
        """Constructor."""
        # Override the default renderer if we were passed one.
        renderer = kwargs.pop('renderer', None)
        if renderer:
            self.renderer = renderer

        if 'choices' not in kwargs:
            kwargs['choices'] = BOOLEAN_CHOICES

        super(BooleanRadioSelect, self).__init__(*args, **kwargs)



from fobi.helpers import validate_submit_value_as

from .conf import get_setting

__title__ = 'fobi.contrib.plugins.form_elements.fields.radio.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SUBMIT_VALUE_AS',)

SUBMIT_VALUE_AS = get_setting('SUBMIT_VALUE_AS')

validate_submit_value_as(SUBMIT_VALUE_AS)



from __future__ import print_function

from django.conf import settings

__title__ = 'fobi.tests.base'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'PRINT_INFO',
    'print_info',
    'fobi_setup',
    'skip',
    'is_fobi_setup_completed',
    'mark_fobi_setup_as_completed',
)

# ****************************************************************************
# **************** Safe User import for Django > 1.5, < 1.8 ******************
# ****************************************************************************
AUTH_USER_MODEL = settings.AUTH_USER_MODEL

# ****************************************************************************
# ****************************************************************************
# ****************************************************************************

PRINT_INFO = True


def print_info(func):
    """Prints some useful info."""
    if not PRINT_INFO:
        return func

    def inner(self, *args, **kwargs):
        """Inner."""
        result = func(self, *args, **kwargs)

        print('\n{0}'.format(func.__name__))
        print('============================')
        if func.__doc__:
            print('""" {0} """'.format(func.__doc__.strip()))
        print('----------------------------')
        if result is not None:
            print(result)
        print('\n')

        return result
    return inner


SKIP = False


def skip(func):
    """Simply skips the test."""
    def inner(self, *args, **kwargs):
        """Inner."""
        if SKIP:
            return
        return func(self, *args, **kwargs)
    return inner


class FobiSetup(object):
    """Setup fobi.

    Basic setup class in order to avoid the fobi test data
    to be initialised multiple times.
    """

    def __init__(self):
        self.is_done = False


fobi_setup = FobiSetup()


def is_fobi_setup_completed():
    """Is fobi setup completed?"""
    return fobi_setup.is_done is True


def mark_fobi_setup_as_completed():
    """Mark fobi setup as completed."""
    fobi_setup.is_done = True



from django import forms
from django.utils.translation import ugettext_lazy as _

from fobi.base import BaseFormFieldPluginForm, get_theme
from fobi.helpers import get_registered_models

from .settings import IGNORED_MODELS

__title__ = 'fobi.contrib.plugins.form_elements.fields.' \
            'select_multiple_model_objects.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SelectMultipleModelObjectsInputForm',)

theme = get_theme(request=None, as_instance=True)


class SelectMultipleModelObjectsInputForm(forms.Form, BaseFormFieldPluginForm):
    """Form for ``SelectMultipleModelObjectsPlugin``."""

    plugin_data_fields = [
        ("label", ""),
        ("name", ""),
        ("model", ""),
        ("help_text", ""),
        ("initial", ""),
        ("required", False)
    ]

    label = forms.CharField(
        label=_("Label"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    name = forms.CharField(
        label=_("Name"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    model = forms.ChoiceField(
        label=_("Model"),
        choices=[],
        required=False,
        widget=forms.widgets.Select(
            attrs={'class': theme.form_element_html_class})
        )
    help_text = forms.CharField(
        label=_("Help text"),
        required=False,
        widget=forms.widgets.Textarea(
            attrs={'class': theme.form_element_html_class}
        )
    )
    initial = forms.CharField(
        label=_("Initial"),
        required=False,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    required = forms.BooleanField(
        label=_("Required"),
        required=False,
        widget=forms.widgets.CheckboxInput(
            attrs={'class': theme.form_element_checkbox_html_class}
        )
    )

    def __init__(self, *args, **kwargs):
        """Constructor.

        In order to avoid static calls to `get_registered_models`.
        """
        super(SelectMultipleModelObjectsInputForm, self).__init__(
            *args, **kwargs
        )
        self.fields['model'].choices = get_registered_models(
            ignore=IGNORED_MODELS
        )



__title__ = 'fobi.contrib.plugins.form_elements.fields.' \
            'range_select.defaults'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'INITIAL',
    'MAX_VALUE',
    'MIN_VALUE',
    'STEP',
)

INITIAL = 50
MIN_VALUE = 0
MAX_VALUE = 100
STEP = 1



from __future__ import print_function

import unittest

from .tests.test_browser_build_dynamic_forms import *
from .tests.test_core import *
from .tests.test_dynamic_forms import *
from .tests.test_sortable_dict import *

__title__ = 'fobi.test'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'


if __name__ == '__main__':
    unittest.main()



from django.forms.fields import IntegerField  # , DecimalField, FloatField
from django.utils.translation import ugettext_lazy as _

from fobi.base import FormFieldPlugin, form_element_plugin_registry, get_theme
from fobi.widgets import NumberInput

from . import UID
from .forms import IntegerInputForm

__title__ = 'fobi.contrib.plugins.form_elements.fields.' \
            'integer.fobi_form_elements'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('IntegerInputPlugin',)

theme = get_theme(request=None, as_instance=True)


class IntegerInputPlugin(FormFieldPlugin):
    """Integer input plugin."""

    uid = UID
    name = _("Integer")
    group = _("Fields")
    form = IntegerInputForm

    def get_form_field_instances(self, request=None):
        """Get form field instances."""
        widget_attrs = {
            'class': theme.form_element_html_class,
            'type': 'number',
            'placeholder': self.data.placeholder,
        }
        kwargs = {
            'label': self.data.label,
            'help_text': self.data.help_text,
            'initial': self.data.initial,
            'required': self.data.required,
        }
        if self.data.max_value:
            kwargs['max_value'] = self.data.max_value
            widget_attrs['max'] = self.data.max_value
        if self.data.min_value:
            kwargs['min_value'] = self.data.min_value
            widget_attrs['min'] = self.data.min_value

        kwargs['widget'] = NumberInput(attrs=widget_attrs)

        return [(self.data.name, IntegerField, kwargs)]


form_element_plugin_registry.register(IntegerInputPlugin)



import unittest

from copy import copy

from django.test import TestCase

from fobi.data_structures import SortableDict

from fobi.tests.base import print_info

__title__ = 'fobi.tests.test_dynamic_forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('FobiDataStructuresTest',)


class FobiDataStructuresTest(TestCase):
    """Tests of django-fobi ``data_structures`` module functionality."""

    def setUp(self):
        """Set up."""
        self.initial = SortableDict(
            [
                ('b', 1),
                ('a', 2),
                ('c', 3),
            ]
        )
        self.expected = SortableDict(
            [
                ('a', 2),
                ('b', 1),
                ('c', 3),
            ]
        )

    @print_info
    def test_01_sortable_dict_move_before_key(self):
        """Test the `fobi.data_structures.SortableDict.move_before_key`."""
        flow = []

        flow.append(copy(self.initial))

        # Expected: a, b, c
        res = self.initial.move_before_key(source_key='a', target_key='b')
        self.assertTrue(res)

        flow.append(copy(self.initial))
        self.assertTrue(self.initial == self.expected)

        return flow

    @print_info
    def test_02_sortable_dict_move_after_key(self):
        """Test the `fobi.data_structures.SortableDict.move_after_key`."""
        flow = []

        flow.append(copy(self.initial))

        # Expected: a, b, c
        res = self.initial.move_after_key(source_key='b', target_key='a')
        self.assertTrue(res)

        flow.append(copy(self.initial))
        self.assertTrue(self.initial == self.expected)

        return flow


if __name__ == '__main__':
    unittest.main()



from six import python_2_unicode_compatible

from django.db import models
from django.utils.translation import ugettext_lazy as _

from mezzanine.core.models import RichText
from mezzanine.pages.models import Page

from .helpers import (
    get_form_template_choices, get_success_page_template_choices
)

__title__ = 'fobi.contrib.apps.mezzanine_integration.models'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('FobiFormPage',)


@python_2_unicode_compatible
class FobiFormPage(Page, RichText):
    """Widget for to Mezzanine.

    :property fobi.models.FormEntry form_entry: Form entry to be rendered.
    :property str template: If given used for rendering the form.
    """

    form_entry = models.ForeignKey('fobi.FormEntry', verbose_name=_("Form"))

    form_template_name = models.CharField(
        _("Form template name"), max_length=255, null=True, blank=True,
        choices=get_form_template_choices(),
        help_text=_("Template to render the form with.")
    )

    hide_form_title = models.BooleanField(
        _("Hide form title"), default=False,
        help_text=_("If checked, no form title is shown.")
    )

    form_title = models.CharField(
        _("Form title"), max_length=255, null=True, blank=True,
        help_text=_("Overrides the default form title.")
    )

    form_submit_button_text = models.CharField(
        _("Submit button text"), max_length=255, null=True, blank=True,
        help_text=_("Overrides the default form submit button text.")
    )

    success_page_template_name = models.CharField(
        _("Success page template name"), max_length=255, null=True, blank=True,
        choices=get_success_page_template_choices(),
        help_text=_("Template to render the success page with.")
    )

    hide_success_page_title = models.BooleanField(
        _("Hide success page title"), default=False,
        help_text=_("If checked, no success page title is shown.")
    )

    success_page_title = models.CharField(
        _("Succes page title"), max_length=255, null=True, blank=True,
        help_text=_("Overrides the default success page title.")
    )

    success_page_text = models.TextField(
        _("Succes page text"), null=True, blank=True,
        help_text=_("Overrides the default success page text.")
    )

    class Meta:
        """Meta class."""

        app_label = 'fobi'
        verbose_name = _("Fobi form")
        verbose_name_plural = _("Fobi forms")
        # db_table = 'fobi_fobiformpage'

    def __str__(self):
        return _('Fobi form')



import logging

import mailchimp

from django.contrib import messages
from django.core.urlresolvers import reverse
from django.shortcuts import redirect
from django.utils.translation import ugettext_lazy as _

from fobi.wizard import SessionWizardView

from .forms import MailchimpAPIKeyForm, MailchimpListIDForm

__title__ = 'fobi.contrib.plugins.form_importers.mailchimp_importer.views'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('MailchimpImporterWizardView',)

logger = logging.getLogger(__name__)


class MailchimpImporterWizardView(SessionWizardView):
    """MailchimpImporterWizardView."""

    form_list = [MailchimpAPIKeyForm, MailchimpListIDForm]

    def get_form_kwargs(self, step):
        """Get form kwargs."""
        if '1' == step:
            data = self.get_cleaned_data_for_step('0') or {}
            api_key = data.get('api_key', None)
            return {'api_key': api_key}
        return {}

    def done(self, form_list, **kwargs):
        # Merging cleaned data into one dict
        cleaned_data = {}
        for form in form_list:
            cleaned_data.update(form.cleaned_data)
        # cleaned_data = self.get_all_cleaned_data()

        # Connecting to mailchimp
        client = mailchimp.Mailchimp(cleaned_data['api_key'])

        # Fetching the form data
        form_data = client.lists.merge_vars(
            id={'list_id': cleaned_data['list_id']}
        )

        # We need the first form only
        try:
            form_data = form_data['data'][0]
        except Exception as err:
            messages.warning(
                self.request,
                _('Selected form could not be imported due errors.')
            )
            return redirect(reverse('fobi.dashboard'))

        # Actually, import the form
        form_entry = self._form_importer.import_data(
            {'name': form_data['name'], 'user': self.request.user},
            form_data['merge_vars']
        )

        redirect_url = reverse(
            'fobi.edit_form_entry', kwargs={'form_entry_id': form_entry.pk}
        )

        messages.info(
            self.request,
            _('Form {0} imported successfully.').format(form_data['name'])
        )

        return redirect("{0}".format(redirect_url))



import gc
import logging
import unittest

from time import sleep

from selenium import webdriver
from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
# from selenium.webdriver.firefox.webdriver import WebDriver
from selenium.webdriver.support.wait import WebDriverWait
from selenium.common.exceptions import WebDriverException

from django.core.management import call_command
from django.core.urlresolvers import reverse
from django.test import LiveServerTestCase
from django.conf import settings

from fobi.models import FormEntry

from fobi.tests.base import print_info, skip  # , BaseBrowserTest
from fobi.tests.helpers import (
    setup_fobi, get_or_create_admin_user, db_clean_up
)
from fobi.tests import constants
from fobi.tests.data import (
    TEST_FORM_ELEMENT_PLUGIN_DATA, TEST_FORM_FIELD_DATA,
    TEST_FORM_HANDLER_PLUGIN_DATA
)

__title__ = 'fobi.tests.test_browser_build_dynamic_forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseFobiBrowserBuldDynamicFormsTest',
    # 'GeneralFobiBrowserBuldDynamicFormsTest',
    # 'FormSpecificFobiBrowserBuldDynamicFormsTest',
    # 'FormElementSpecificFobiBrowserBuldDynamicFormsTest',
    # 'FormHandlerSpecificFobiBrowserBuldDynamicFormsTest',
)

logger = logging.getLogger(__name__)

TIMEOUT = 4
WAIT = False
WAIT_FOR = 0


class BaseFobiBrowserBuldDynamicFormsTest(LiveServerTestCase):
    """Browser tests django-fobi bulding forms functionality.

    Backed up by selenium. This test is based on the bootstrap3 theme.
    """

    cleans_up_after_itself = True
    try:
        LIVE_SERVER_URL = settings.LIVE_SERVER_URL
    except Exception as e:
        LIVE_SERVER_URL = None

    def tearDown(self):
        """Tear down."""
        super(BaseFobiBrowserBuldDynamicFormsTest, self).tearDown()
        call_command('flush', verbosity=0, interactive=False,
                     reset_sequences=False,
                     allow_cascade=False,
                     inhibit_post_migrate=False)
        gc.collect()

    @classmethod
    def setUpClass(cls):
        """Set up class."""
        # cls.selenium = WebDriver()
        firefox_bin_path = getattr(settings, 'FIREFOX_BIN_PATH', None)
        if firefox_bin_path:
            binary = FirefoxBinary(firefox_bin_path)
            cls.selenium = webdriver.Firefox(firefox_binary=binary)
        else:
            cls.selenium = webdriver.Firefox()

        setup_fobi(fobi_sync_plugins=True)
        # user = get_or_create_admin_user()
        # create_form_with_entries(user)

        super(BaseFobiBrowserBuldDynamicFormsTest, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        """Tear down class."""
        try:
            cls.selenium.quit()
        except Exception as err:
            print(err)

        super(BaseFobiBrowserBuldDynamicFormsTest, cls).tearDownClass()
        call_command('flush', verbosity=0, interactive=False,
                     reset_sequences=False,
                     allow_cascade=False,
                     inhibit_post_migrate=False)
        gc.collect()

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++ Internals +++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++ General +++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    def _get_live_server_url(self):
        """Get live server URL."""
        return self.LIVE_SERVER_URL \
            if self.LIVE_SERVER_URL \
            else self.live_server_url

    def _authenticate(self):
        """Authenticate."""
        # Make sure the user exists
        user = get_or_create_admin_user()

        self.selenium.get(
            '{0}{1}'.format(
                self._get_live_server_url(),
                reverse('auth_login')
            )
        )
        self.selenium.maximize_window()
        username_input = self.selenium.find_element_by_name("username")
        username_input.send_keys(constants.FOBI_TEST_USER_USERNAME)
        password_input = self.selenium.find_element_by_name("password")
        password_input.send_keys(constants.FOBI_TEST_USER_PASSWORD)
        self.selenium.find_element_by_xpath('//button[@type="submit"]').click()

        # Wait until the list view opens
        WebDriverWait(self.selenium, timeout=TIMEOUT).until(
            # lambda driver: driver.find_element_by_id('id_main')
            lambda driver: driver.find_element_by_xpath(
                '//body[contains(@class, "theme")]'
            )
        )

    def _sleep(self, wait=WAIT_FOR):
        """Sleep."""
        if WAIT and wait:
            self.selenium.implicitly_wait(wait)

    def _click(self, element):
        """Click on any element."""
        self.selenium.execute_script("$(arguments[0]).click();", element)

    def _go_to_dashboard(self, wait=WAIT_FOR):
        """Go to dashboard."""
        # Authenticate user.
        self._authenticate()

        # Open the dashboard.
        url = reverse('fobi.dashboard')
        self.selenium.get('{0}{1}'.format(self._get_live_server_url(), url))

        # Wait until the edit widget form opens
        WebDriverWait(self.selenium, timeout=TIMEOUT).until(
            lambda driver: driver.find_element_by_xpath(
                '//body[contains(@class, "theme-bootstrap3")]'
            )
        )

        self._sleep(wait)

    def _scroll_to_element(self, form_element, simple=False):
        """Scroll to element."""
        coordinates = form_element.location_once_scrolled_into_view
        if simple:
            return

        x = coordinates.get('x', 0)
        y = coordinates.get('y', 0)
        self.selenium.execute_script(
            "window.scrollTo({0}, {1});".format(x, y)
        )
        self.selenium.execute_script(
            "window.scrollBy({0}, {1});".format(0, -100)
        )

    def _scroll_to(self, x, y):
        """Scroll to."""
        self.selenium.execute_script(
            "window.scrollTo({0}, {1});".format(x, y)
        )

    def _scroll_by(self, x, y):
        """Scroll by."""
        self.selenium.execute_script(
            "window.scrollBy({0}, {1});".format(x, y)
        )

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # ++++++++++++++++++++++ Form related +++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    def _test_add_form(self, wait=WAIT_FOR):
        """Test add form."""
        # First open the dashboard
        self._go_to_dashboard()

        # Follow the create form link.
        # Click the button to go to dashboard edit
        self.selenium.find_element_by_xpath(
            '//a[contains(@class, "list-group-item")]'
        ).click()

        # Wait until the dashboard edit view opens
        WebDriverWait(self.selenium, timeout=TIMEOUT).until(
            # lambda driver: driver.find_element_by_id('id_main')
            lambda driver: driver.find_element_by_xpath(
                '//body[contains(@class, "theme-bootstrap3")]'
            )
        )

        form_data = {
            'name': constants.TEST_FORM_NAME,
            # 'user': get_or_create_admin_user(),
            # 'slug': constants.TEST_FORM_SLUG,
            'is_public': True,
            'success_page_title': "Success page title",
            'success_page_message': "Success page message",
        }

        if form_data:
            for field_name, field_value in form_data.items():
                field_input = self.selenium.find_element_by_name(field_name)
                field_input.send_keys(field_value)

        # Click add widget button
        self.selenium.find_element_by_xpath('//button[@type="submit"]').click()

        logger.debug(
            """//div[contains(text(), 'Form {0} was created successfully.') and contains(@class, "alert-info")]""".format(
                constants.TEST_FORM_NAME
            )
        )

        # Wait until the fobi page opens with the form element in
        WebDriverWait(self.selenium, timeout=TIMEOUT).until(
            lambda driver: driver.find_element_by_xpath(
                """//div[contains(text(), 'Form {0} was created successfully.') and contains(@class, "alert-info")]""".format(
                    constants.TEST_FORM_NAME
                )
            )
        )

        self._sleep(wait)

    def _get_form(self):
        """Get form object."""
        return FormEntry._default_manager.get(name=constants.TEST_FORM_NAME)

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # ++++++++++++++++++++ Form element specific ++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    def _add_form_element(self, form_element_name, form_element_data,
                          wait=WAIT_FOR):
        """Add a single plugin to the form.

        At this stage we should be in the edit form entry interface.
        """
        # Click the add form element button to add a new form element to the
        # form.
        add_form_element_link = self.selenium.find_element_by_xpath(
            """//a[contains(text(), 'Choose form element to add') and contains(@class, "dropdown-toggle")]"""
        )

        self._scroll_to(0, 0)
        add_form_element_link.click()

        # Find the parent element
        add_form_element_parent_container = \
            add_form_element_link.find_element_by_xpath('..')

        # Find the container of the available form elements
        add_form_element_available_elements_container = \
            add_form_element_parent_container.find_element_by_xpath(
                '//ul[contains(@class, "dropdown-menu")]'
            )

        # Click on the element we want
        form_element_to_add = \
            add_form_element_available_elements_container.find_element_by_xpath(
                '//a[text()="{0}"]'.format(form_element_name)
            )

        self._scroll_to_element(form_element_to_add, simple=True)
        self._scroll_by(0, -150)
        form_element_to_add.click()

        # Adding form data
        if form_element_data:
            # Wait until the add widget view opens
            WebDriverWait(self.selenium, timeout=TIMEOUT).until(
                lambda driver: driver.find_element_by_xpath(
                    """//h1[contains(text(), 'Add "{0}" element to the form')]""".format(
                        form_element_name
                    )
                )
            )

            for field_name, field_value in form_element_data.items():
                field_input = self.selenium.find_element_by_name(field_name)
                field_input.send_keys(field_value)

            # Click add widget button
            self.selenium.find_element_by_xpath(
                '//button[@type="submit"]'
            ).click()

        logger.debug(form_element_name)

        # Wait until the fobi page opens with the form element in.
        WebDriverWait(self.selenium, timeout=TIMEOUT).until(
            lambda driver: driver.find_element_by_xpath(
                """//div[contains(text(), 'The form element plugin "{0}" was added successfully.') and contains(@class, "alert-info")]""".format(
                    form_element_name
                )
            )
        )

    def _test_add_form_elements(self, create_form=False):
        """Test of adding multiple form elements.

        At this point form should be created.
        """
        if create_form:
            # Adding a form first
            self._test_add_form()

        # One by one adding form elements with data.
        # Example follows:
        #
        # self._add_form_element(
        #     force_text(BooleanSelectPlugin.name),
        #     {
        #         'label': "Test boolean",
        #         'name': "test_boolean",
        #         'help_text': "Lorem ipsum boolean",
        #         'required': False,
        #     }
        # )

        for plugin_name, plugin_data in TEST_FORM_ELEMENT_PLUGIN_DATA.items():
            # Add form element to the form
            self._add_form_element(plugin_name, plugin_data)

    def _remove_form_element(self, form_element_name, form_element_data,
                             wait=WAIT_FOR):
        """Add a single plugin to the form.

        At this stage we should be in the edit form entry interface.
        """
        # Get the label of the given form element in order to delete it later
        # from the form.
        delete_form_element_label = self.selenium.find_element_by_xpath(
            """//label[contains(text(), '({0})') and contains(@class, "control-label")]""".format(
                form_element_name
            )
        )

        # Get the parent of the label
        delete_form_element_label_parent_container = \
            delete_form_element_label.find_element_by_xpath('..')

        # Click the add form element button to add a new form element to the
        # form.
        delete_form_element_link = \
            delete_form_element_label_parent_container.find_element_by_partial_link_text(
                'Delete'
            )
        # delete_form_element_link.click()
        self._click(delete_form_element_link)

        logger.debug(form_element_name)

        # Wait until the fobi page opens with the form element in.
        WebDriverWait(self.selenium, timeout=TIMEOUT).until(
            lambda driver: driver.find_element_by_xpath(
                """//div[contains(text(), 'The form element plugin "{0}" was deleted successfully.') and contains(@class, "alert-info")]""".format(
                    form_element_name
                )
            )
        )

    def _test_remove_form_elements(self):
        """Test of removing multiple form elements.

        At this point form should be created.
        """
        for plugin_name, plugin_data in TEST_FORM_ELEMENT_PLUGIN_DATA.items():
            # Add form element to the form
            self._remove_form_element(plugin_name, plugin_data)

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # ++++++++++++++++++++ Form handler specific ++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    def _add_form_handler(self, form_handler_name, form_handler_data,
                          wait=WAIT_FOR):
        """Add a single handler to the form.

        At this stage we should be in the edit form entry interface.
        """
        # This part is necessary, since we need to activate the
        # tab with form handlers. Otherwise Selenium raises
        # an exception about non-visible element on the page
        # that we're trying to fetch.
        form_handlers_tab_link = self.selenium.find_element_by_xpath(
            """//a[@href="#tab-form-handlers"]"""
        )
        form_handlers_tab_link.click()

        # Click the add form element button to add a new form element to the
        # form.
        add_form_handler_link = self.selenium.find_element_by_xpath(
            """//a[contains(text(), 'Choose form handler to add') and contains(@class, "dropdown-toggle")]"""
        )
        add_form_handler_link.click()

        # Find the parent element
        add_form_handler_parent_container = \
            add_form_handler_link.find_element_by_xpath('..')

        # Find the container of the available form elements
        add_form_handler_available_elements_container = \
            add_form_handler_parent_container.find_element_by_xpath(
                '//ul[contains(@class, "dropdown-menu")]'
            )

        # Click on the element we want
        form_handler_to_add = \
            add_form_handler_available_elements_container.find_element_by_xpath(
                '//a[text()="{0}"]'.format(form_handler_name)
            )
        form_handler_to_add.click()

        # If has config, there's a need to perform some extra tests.
        if form_handler_data:
            # Wait until the add widget view opens
            WebDriverWait(self.selenium, timeout=TIMEOUT).until(
                lambda driver: driver.find_element_by_xpath(
                    """//h1[contains(text(), 'Add "{0}" handler to the form')]""".format(
                        form_handler_name
                    )
                )
            )

            # Config
            for field_name, field_value in form_handler_data.items():
                field_input = self.selenium.find_element_by_name(field_name)
                field_input.send_keys(field_value)

            # Click add widget button
            self.selenium.find_element_by_xpath(
                '//button[@type="submit"]'
            ).click()

        # Wait until the fobi page opens with the form element in.
        WebDriverWait(self.selenium, timeout=TIMEOUT).until(
            lambda driver: driver.find_element_by_xpath(
                """//div[contains(text(), 'The form handler plugin "{0}" was added successfully.') and contains(@class, "alert-info")]""".format(
                    form_handler_name
                )
            )
        )

    def _test_add_form_handlers(self, create_form=False):
        """Test of adding multiple form handlers.

        At this point form should be created.
        """
        if create_form:
            # Adding a form first
            self._test_add_form()

        # One by one adding form elements with data.
        # Example follows:
        #
        # self._add_form_element(
        #     force_text(BooleanSelectPlugin.name),
        #     {
        #         'label': "Test boolean",
        #         'name': "test_boolean",
        #         'help_text': "Lorem ipsum boolean",
        #         'required': False,
        #     }
        # )

        for plugin_name, plugin_data in TEST_FORM_HANDLER_PLUGIN_DATA.items():
            # Add form element to the form
            self._add_form_handler(plugin_name, plugin_data)

    def _remove_form_handler(self, form_handler_name, form_handler_data,
                             wait=WAIT_FOR):
        """Remove a single handler from the form.

        At this stage we should be in the edit form entry interface.
        """
        # This part is necessary, since we need to activate the
        # tab with form handlers. Otherwise Selenium raises
        # an exception about non-visible element on the page
        # that we're trying to fetch.
        form_handlers_tab_link = self.selenium.find_element_by_xpath(
            """//a[@href="#tab-form-handlers"]"""
        )
        form_handlers_tab_link.click()

        # Get the label of the given form element in order to delete it later
        # from the form.
        delete_form_handler_label = self.selenium.find_element_by_xpath(
            """//td[contains(text(), '{0}')]""".format(form_handler_name)
        )

        # Get the parent of the label
        delete_form_handler_label_parent_container = \
            delete_form_handler_label.find_element_by_xpath('..')

        # Click the add form element button to add a new form element to the
        # form.
        delete_form_handler_link = \
            delete_form_handler_label_parent_container.find_element_by_partial_link_text(
                'Delete'
            )
        delete_form_handler_link.click()

        logger.debug(form_handler_name)

        # Wait until the fobi page opens with the form element in.
        WebDriverWait(self.selenium, timeout=TIMEOUT).until(
            lambda driver: driver.find_element_by_xpath(
                """//div[contains(text(), 'The form handler plugin "{0}" was deleted successfully.') and contains(@class, "alert-info")]""".format(
                    form_handler_name
                )
            )
        )

    def _test_remove_form_handlers(self):
        """Test of removing multiple form handlers.

        At this point form should be created.
        """
        for plugin_name, plugin_data in TEST_FORM_HANDLER_PLUGIN_DATA.items():
            # Remove form handler from the form
            self._remove_form_handler(plugin_name, plugin_data)

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++ Tests +++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++ General +++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    @skip
    @print_info
    def test_1001_open_dashboard(self):
        """Test open dashboard."""
        self._go_to_dashboard()


# class GeneralFobiBrowserBuldDynamicFormsTest(
#         BaseFobiBrowserBuldDynamicFormsTest):
#     """General tests."""
#
#
# class FormSpecificFobiBrowserBuldDynamicFormsTest(
#         BaseFobiBrowserBuldDynamicFormsTest):
#     """Form specific tests."""

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # ++++++++++++++++++++++ Form specific ++++++++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    @skip
    @print_info
    def test_2001_add_form(self):
        """Test add a new form."""
        # Clean up database
        db_clean_up()

        self._test_add_form(wait=WAIT_FOR)

        # Make sure the success message is there
        # self.selenium.find_element_by_xpath(
        #     """//div[text()='Form {0} was created successfully.']""".format(
        #         constants.TEST_FORM_NAME
        #     )
        # )

    @skip
    @print_info
    def test_2002_edit_form(self):
        """Test edit a form."""
        # TODO

    @skip
    @print_info
    def test_2003_delete_form(self):
        """Test delete a form."""
        # TODO

    @skip
    @print_info
    def test_2004_submit_form(self, wait=WAIT_FOR):
        """Test submit form."""
        # Clean up database
        db_clean_up()

        # Add form elements
        self._test_add_form_elements(create_form=True)

        # self._sleep(wait)

        # Getting a form object
        form = self._get_form()

        # Getting the form URL
        url = reverse('fobi.view_form_entry', args=[form.slug])
        self.selenium.get('{0}{1}'.format(self._get_live_server_url(), url))

        # Wait until the edit widget form opens
        WebDriverWait(self.selenium, timeout=TIMEOUT).until(
            lambda driver: driver.find_element_by_xpath(
                '//body[contains(@class, "theme-bootstrap3")]'
            )
        )

        for field_name, field_value in TEST_FORM_FIELD_DATA.items():
            field_input = self.selenium.find_element_by_name(field_name)
            field_input.send_keys(field_value)

        self._sleep(2)

        # Click add widget button
        self.selenium.find_element_by_xpath('//button[@type="submit"]').click()

        # Wait until the submit success page opens a clear success message.
        WebDriverWait(self.selenium, timeout=TIMEOUT).until(
            lambda driver: driver.find_element_by_xpath(
                """//div[contains(text(), 'Form {0} was submitted successfully.') and contains(@class, "alert-info")]""".format(
                    constants.TEST_FORM_NAME
                )
            )
        )

        self._sleep(wait)


# class FormElementSpecificFobiBrowserBuldDynamicFormsTest(
#         BaseFobiBrowserBuldDynamicFormsTest):
#     """Form element specific."""

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # ++++++++++++++++++++ Form element specific ++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    @skip
    @print_info
    def test_3001_add_form_elements(self, wait=WAIT_FOR):
        """Test adding form elements."""
        # Clean up database
        db_clean_up()

        self._test_add_form_elements(create_form=True)

        self._sleep(wait)

    @skip
    @print_info
    def test_3002_remove_form_elements(self):
        """Test remove form element."""
        # Clean up database
        db_clean_up()

        self._test_add_form_elements(create_form=True)

        self._test_remove_form_elements()

    @skip
    @print_info
    def test_3003_edit_form_elements(self):
        """Test edit form element."""


# class FormHandlerSpecificFobiBrowserBuldDynamicFormsTest(
#         BaseFobiBrowserBuldDynamicFormsTest):
#     """Form handler specific."""

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # ++++++++++++++++++++ Form handler specific ++++++++++++++++++++
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    @skip
    @print_info
    def test_4001_add_form_handlers(self, wait=WAIT_FOR):
        """Test of adding a single form handler.

        At this point, if form isn't created, it should be.
        """
        # Clean up database
        db_clean_up()

        self._test_add_form_handlers(create_form=True)

        self._sleep(wait)

    @skip
    @print_info
    def test_4002_remove_form_handlers(self):
        """Test remove form handler."""
        # Clean up database
        db_clean_up()

        self._test_add_form_handlers(create_form=True)

        self._test_remove_form_handlers()

    @skip
    @print_info
    def test_4003_edit_form_handlers(self):
        """Test edit form handler."""


if __name__ == '__main__':
    unittest.main()



__title__ = 'fobi.exceptions'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'BaseException',
    'ImproperlyConfigured',
    'InvalidRegistryItemType',
    'DoesNotExist',
    'ThemeDoesNotExist',
    'PluginDoesNotExist',
    'FormElementPluginDoesNotExist',
    'FormHandlerPluginDoesNotExist',
    'FormWizardHandlerPluginDoesNotExist',
    'NoDefaultThemeSet',
    'FormPluginError',
    'FormElementPluginError',
    'FormHandlerPluginError',
    'FormCallbackError',
)


class BaseException(Exception):
    """Base exception."""


class ImproperlyConfigured(BaseException):
    """Improperly configured.

    Exception raised when developer didn't configure/write the code properly.
    """


class InvalidRegistryItemType(ValueError, BaseException):
    """Invalid registry item type.

    Raised when an attempt is made to register an item in the registry which
    does not have a proper type.
    """


class DoesNotExist(BaseException):
    """Raised when something does not exist."""


class ThemeDoesNotExist(DoesNotExist):
    """Raised when no theme with given uid can be found."""


class PluginDoesNotExist(DoesNotExist):
    """Raised when no plugin with given uid can be found."""


class FormElementPluginDoesNotExist(PluginDoesNotExist):
    """Raised when no form element plugin with given uid can be found."""


class FormHandlerPluginDoesNotExist(PluginDoesNotExist):
    """Raised when no form handler plugin with given uid can be found."""


class FormWizardHandlerPluginDoesNotExist(PluginDoesNotExist):
    """FormWizardHandlerPlugin does not exist.

    Raised when no form wizard handler plugin with given uid can be found.
    """


class NoDefaultThemeSet(ImproperlyConfigured):
    """Raised when no active theme is chosen."""


class FormPluginError(BaseException):
    """Base error for form elements and handlers."""


class FormElementPluginError(FormPluginError):
    """Raised when form element plugin error occurs."""


class FormHandlerPluginError(FormPluginError):
    """Raised when form handler plugin error occurs."""


class FormCallbackError(FormPluginError):
    """Raised when form callback error occurs."""



from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from fobi.views import (
    add_form_element_entry,
    add_form_handler_entry,
    add_form_wizard_form_entry,
    add_form_wizard_handler_entry,
    create_form_entry,
    create_form_wizard_entry,
    dashboard,
    delete_form_element_entry,
    delete_form_entry,
    delete_form_handler_entry,
    delete_form_wizard_entry,
    delete_form_wizard_form_entry,
    delete_form_wizard_handler_entry,
    edit_form_element_entry,
    edit_form_entry,
    edit_form_handler_entry,
    edit_form_wizard_entry,
    edit_form_wizard_handler_entry,
    export_form_entry,
    form_importer,
    form_wizards_dashboard,
    import_form_entry,
)

__title__ = 'fobi.urls.edit'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('urlpatterns',)


urlpatterns = [
    # ***********************************************************************
    # **************************** Form entry CUD ***************************
    # ***********************************************************************

    # Create form entry
    url(_(r'^forms/create/$'),
        view=create_form_entry,
        name='fobi.create_form_entry'),

    # Edit form entry
    url(_(r'^forms/edit/(?P<form_entry_id>\d+)/$'),
        edit_form_entry,
        name='fobi.edit_form_entry'),

    # Delete form entry
    url(_(r'^forms/delete/(?P<form_entry_id>\d+)/$'),
        delete_form_entry,
        name='fobi.delete_form_entry'),

    # ***********************************************************************
    # ************************ Form entry add-ons ***************************
    # ***********************************************************************

    # Export form entry
    url(_(r'^forms/export/(?P<form_entry_id>\d+)/$'),
        export_form_entry,
        name='fobi.export_form_entry'),

    # Import form entry
    url(_(r'^forms/import/$'),
        import_form_entry,
        name='fobi.import_form_entry'),

    # Form importers
    url(_(r'^forms/importer/(?P<form_importer_plugin_uid>[\w_\-]+)/$'),
        form_importer,
        name='fobi.form_importer'),

    # ***********************************************************************
    # *********************** Form element entry CUD ************************
    # ***********************************************************************

    # Add form element entry
    url(_(r'^forms/elements/add/(?P<form_entry_id>\d+)/'
          r'(?P<form_element_plugin_uid>[\w_\-]+)/$'),
        add_form_element_entry,
        name='fobi.add_form_element_entry'),

    # Edit form element entry
    url(_(r'^forms/elements/edit/(?P<form_element_entry_id>\d+)/$'),
        edit_form_element_entry,
        name='fobi.edit_form_element_entry'),

    # Delete form element entry
    url(_(r'^forms/elements/delete/(?P<form_element_entry_id>\d+)/$'),
        delete_form_element_entry,
        name='fobi.delete_form_element_entry'),

    # ***********************************************************************
    # *********************** Form handler entry CUD ************************
    # ***********************************************************************

    # Add form handler entry
    url(_(r'^forms/handlers/add/(?P<form_entry_id>\d+)/'
          r'(?P<form_handler_plugin_uid>[\w_\-]+)/$'),
        add_form_handler_entry,
        name='fobi.add_form_handler_entry'),

    # Edit form handler entry
    url(_(r'^forms/handlers/edit/(?P<form_handler_entry_id>\d+)/$'),
        edit_form_handler_entry,
        name='fobi.edit_form_handler_entry'),

    # Delete form handler entry
    url(_(r'^forms/handlers/delete/(?P<form_handler_entry_id>\d+)/$'),
        delete_form_handler_entry,
        name='fobi.delete_form_handler_entry'),

    # ***********************************************************************
    # ************************ Form wizard entry CUD ************************
    # ***********************************************************************

    # Create form wizard entry
    url(_(r'^wizard/create/$'),
        view=create_form_wizard_entry,
        name='fobi.create_form_wizard_entry'),

    # Edit form wizard entry
    url(_(r'^wizard/edit/(?P<form_wizard_entry_id>\d+)/$'),
        edit_form_wizard_entry,
        name='fobi.edit_form_wizard_entry'),

    # Delete form wizard entry
    url(_(r'^wizard/delete/(?P<form_wizard_entry_id>\d+)/$'),
        delete_form_wizard_entry,
        name='fobi.delete_form_wizard_entry'),

    # ***********************************************************************
    # ******************** Form wizard form entry CUD ***********************
    # ***********************************************************************

    # Add form wizard form entry
    url(_(r'^wizard/forms/add/(?P<form_wizard_entry_id>\d+)/'
          r'(?P<form_entry_id>[\w_\-]+)/$'),
        add_form_wizard_form_entry,
        name='fobi.add_form_wizard_form_entry'),

    # # Edit form wizard form entry
    # url(_(r'^wizard/forms/edit/(?P<form_element_entry_id>\d+)/$'),
    #     edit_form_wizard_form_entry,
    #     name='fobi.edit_form_wizard_form_entry'),
    #
    # Delete form wizard form entry
    url(_(r'^wizard/elements/delete/(?P<form_wizard_form_entry_id>\d+)/$'),
        delete_form_wizard_form_entry,
        name='fobi.delete_form_wizard_form_entry'),

    # ***********************************************************************
    # ******************* Form wizard handler entry CUD *********************
    # ***********************************************************************

    # Add form wizard handler entry
    url(_(r'^wizard/handlers/add/(?P<form_wizard_entry_id>\d+)/'
          r'(?P<form_wizard_handler_plugin_uid>[\w_\-]+)/$'),
        add_form_wizard_handler_entry,
        name='fobi.add_form_wizard_handler_entry'),

    # Edit form wizard handler entry
    url(_(r'^wizard/handlers/edit/(?P<form_wizard_handler_entry_id>\d+)/$'),
        edit_form_wizard_handler_entry,
        name='fobi.edit_form_wizard_handler_entry'),

    # Delete form wizard handler entry
    url(_(r'^wizard/handlers/delete/(?P<form_wizard_handler_entry_id>\d+)/$'),
        delete_form_wizard_handler_entry,
        name='fobi.delete_form_wizard_handler_entry'),

    # ***********************************************************************
    # ****************************** Dashboard ******************************
    # ***********************************************************************

    # Forms dashboard
    url(_(r'^$'), view=dashboard, name='fobi.dashboard'),

    # Form wizards dashboard
    url(_(r'^wizards/$'),
        view=form_wizards_dashboard,
        name='fobi.form_wizards_dashboard'),
]



from django import forms
from django.utils.translation import ugettext_lazy as _

from fobi.base import BaseFormFieldPluginForm, get_theme
from fobi.helpers import get_registered_models

from .settings import IGNORED_MODELS

__title__ = 'fobi.contrib.plugins.form_elements.fields.' \
            'hidden_model_object.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('HiddenModelObjectInputForm',)

theme = get_theme(request=None, as_instance=True)


class HiddenModelObjectInputForm(forms.Form, BaseFormFieldPluginForm):
    """Form for ``HiddenModelObjectPlugin``."""

    plugin_data_fields = [
        ("label", ""),
        ("name", ""),
        ("model", ""),
        ("help_text", ""),
        ("initial", ""),
        ("required", False)
    ]

    label = forms.CharField(
        label=_("Label"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    name = forms.CharField(
        label=_("Name"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    model = forms.ChoiceField(
        label=_("Model"),
        choices=[],
        required=False,
        widget=forms.widgets.Select(
            attrs={'class': theme.form_element_html_class}
        )
    )
    help_text = forms.CharField(
        label=_("Help text"),
        required=False,
        widget=forms.widgets.Textarea(
            attrs={'class': theme.form_element_html_class}
        )
    )
    initial = forms.CharField(
        label=_("Initial"),
        required=False,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    required = forms.BooleanField(
        label=_("Required"),
        required=False,
        widget=forms.widgets.CheckboxInput(
            attrs={'class': theme.form_element_checkbox_html_class}
        )
    )

    def __init__(self, *args, **kwargs):
        """In order to avoid static calls to `get_registered_models`."""
        super(HiddenModelObjectInputForm, self).__init__(*args, **kwargs)
        self.fields['model'].choices = get_registered_models(
            ignore=IGNORED_MODELS
        )



from __future__ import absolute_import

from mimetypes import guess_type
import os

from six import string_types

from django.conf import settings
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from fobi.base import (
    # General
    get_processed_form_data,
    get_processed_form_wizard_data,

    # Form
    FormHandlerPlugin,
    form_handler_plugin_registry,

    # Form wizard
    FormWizardHandlerPlugin,
    form_wizard_handler_plugin_registry,
    # get_form_wizard_handler_plugin_widget
)
from fobi.helpers import (
    safe_text,
    extract_file_path,
    get_form_element_entries_for_form_wizard_entry
)

from . import UID
from .forms import MailForm
from .helpers import send_mail
from .settings import MULTI_EMAIL_FIELD_VALUE_SPLITTER

__title__ = 'fobi.contrib.plugins.form_handlers.mail.fobi_form_handlers'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'MailHandlerPlugin',
    'MailWizardHandlerPlugin',
)

# *****************************************************************************
# **************************** Form handler ***********************************
# *****************************************************************************


class MailHandlerPlugin(FormHandlerPlugin):
    """Mail handler plugin.

    Sends emails to the person specified. Should be executed before
    ``db_store`` and ``http_repost`` plugins.
    """

    uid = UID
    name = _("Mail")
    form = MailForm

    def run(self, form_entry, request, form, form_element_entries=None):
        """Run.

        :param fobi.models.FormEntry form_entry: Instance of
            ``fobi.models.FormEntry``.
        :param django.http.HttpRequest request:
        :param django.forms.Form form:
        :param iterable form_element_entries: Iterable of
            ``fobi.models.FormElementEntry`` objects.
        """
        base_url = 'http{secure}://{host}'.format(
            secure=('s' if request.is_secure() else ''),
            host=request.get_host()
        )

        # Clean up the values, leave our content fields and empty values.
        field_name_to_label_map, cleaned_data = get_processed_form_data(
            form,
            form_element_entries
        )

        rendered_data = []
        for key, value in cleaned_data.items():
            if value and isinstance(value, string_types) \
                     and value.startswith(settings.MEDIA_URL):
                cleaned_data[key] = '{base_url}{value}'.format(
                    base_url=base_url, value=value
                )
            label = field_name_to_label_map.get(key, key)
            rendered_data.append('{0}: {1}\n'.format(
                safe_text(label), safe_text(cleaned_data[key]))
            )

        files = self._prepare_files(request, form)

        # Handling more than one email address
        if isinstance(self.data.to_email, (list, tuple)):
            to_email = self.data.to_email
        else:
            # Assume that it's string
            to_email = self.data.to_email.split(
                MULTI_EMAIL_FIELD_VALUE_SPLITTER
            )

        send_mail(
            safe_text(self.data.subject),
            "{0}\n\n{1}".format(
                safe_text(self.data.body),
                ''.join(rendered_data)
            ),
            self.data.from_email,
            to_email,
            fail_silently=False,
            attachments=files.values()
        )

    def _prepare_files(self, request, form):
        """Prepares the files for being attached to the mail message."""
        files = {}

        def process_path(file_path, imf):
            """Processes the file path and the file."""
            if file_path:
                # if file_path.startswith(settings.MEDIA_URL):
                #     file_path = file_path[1:]
                # file_path = settings.PROJECT_DIR('../{0}'.format(file_path))
                file_path = file_path.replace(
                    settings.MEDIA_URL,
                    os.path.join(settings.MEDIA_ROOT, '')
                )
                mime_type = guess_type(imf.name)
                files[field_name] = (
                    imf.name,
                    ''.join([c for c in imf.chunks()]),
                    mime_type[0] if mime_type else ''
                )

        for field_name, imf in request.FILES.items():
            try:
                file_path = form.cleaned_data.get(field_name, '')
                process_path(file_path, imf)
            except Exception as err:
                file_path = extract_file_path(imf.name)
                process_path(file_path, imf)

        return files

    def plugin_data_repr(self):
        """Human readable representation of plugin data.

        :return string:
        """
        to_email = None
        # Handling more than one email address
        if isinstance(self.data.to_email, (list, tuple)):
            to_email = '{0} '.format(MULTI_EMAIL_FIELD_VALUE_SPLITTER).join(
                self.data.to_email
            )
        else:
            # Assume that it's string
            to_email = self.data.to_email

        context = {
            'to_name': safe_text(self.data.to_name),
            'to_email': to_email,
            'subject': safe_text(self.data.subject),
        }
        return render_to_string('mail/plugin_data_repr.html', context)


form_handler_plugin_registry.register(MailHandlerPlugin)

# *****************************************************************************
# ************************ Form wizard handler ********************************
# *****************************************************************************


class MailWizardHandlerPlugin(FormWizardHandlerPlugin):
    """Mail wizard handler plugin.

    Sends emails to the person specified. Should be executed before
    ``db_store`` and ``http_repost`` plugins.
    """

    uid = UID
    name = _("Mail")
    form = MailForm

    def run(self, form_wizard_entry, request, form_list, form_wizard,
            form_element_entries=None):
        """Run.

        :param fobi.models.FormWizardEntry form_wizard_entry: Instance
            of :class:`fobi.models.FormWizardEntry`.
        :param django.http.HttpRequest request:
        :param list form_list: List of :class:`django.forms.Form` instances.
        :param fobi.wizard.views.dynamic.DynamicWizardView form_wizard:
            Instance of :class:`fobi.wizard.views.dynamic.DynamicWizardView`.
        :param iterable form_element_entries: Iterable of
            ``fobi.models.FormElementEntry`` objects.
        """
        base_url = 'http{secure}://{host}'.format(
            secure=('s' if request.is_secure() else ''),
            host=request.get_host()
        )

        if not form_element_entries:
            form_element_entries = \
                get_form_element_entries_for_form_wizard_entry(
                    form_wizard_entry
                )

        # Clean up the values, leave our content fields and empty values.
        field_name_to_label_map, cleaned_data = get_processed_form_wizard_data(
            form_wizard,
            form_list,
            form_element_entries
        )

        rendered_data = []
        for key, value in cleaned_data.items():
            if value and isinstance(value, string_types) \
                     and value.startswith(settings.MEDIA_URL):
                cleaned_data[key] = '{base_url}{value}'.format(
                    base_url=base_url, value=value
                )
            label = field_name_to_label_map.get(key, key)
            rendered_data.append('{0}: {1}\n'.format(
                safe_text(label), safe_text(cleaned_data[key]))
            )

        files = self._prepare_files(request, form_list)

        # Handling more than one email address
        if isinstance(self.data.to_email, (list, tuple)):
            to_email = self.data.to_email
        else:
            # Assume that it's string
            to_email = self.data.to_email.split(
                MULTI_EMAIL_FIELD_VALUE_SPLITTER
            )

        send_mail(
            safe_text(self.data.subject),
            "{0}\n\n{1}".format(
                safe_text(self.data.body),
                ''.join(rendered_data)
            ),
            self.data.from_email,
            to_email,
            fail_silently=False,
            attachments=files.values()
        )

    def _prepare_files(self, request, form_list):
        """Prepares the files for being attached to the mail message."""
        files = {}

        def process_path(file_path, imf):
            """Processes the file path and the file."""
            if file_path:
                # if file_path.startswith(settings.MEDIA_URL):
                #     file_path = file_path[1:]
                # file_path = settings.PROJECT_DIR('../{0}'.format(file_path))
                file_path = file_path.replace(
                    settings.MEDIA_URL,
                    os.path.join(settings.MEDIA_ROOT, '')
                )
                mime_type = guess_type(imf.name)
                files[field_name] = (
                    imf.name,
                    ''.join([c for c in imf.chunks()]),
                    mime_type[0] if mime_type else ''
                )

        for form in form_list:
            for field_name, imf in request.FILES.items():
                try:
                    file_path = form.cleaned_data.get(field_name, '')
                    process_path(file_path, imf)
                except Exception as err:
                    file_path = extract_file_path(imf.name)
                    process_path(file_path, imf)

        return files

    def plugin_data_repr(self):
        """Human readable representation of plugin data.

        :return string:
        """
        to_email = None
        # Handling more than one email address
        if isinstance(self.data.to_email, (list, tuple)):
            to_email = '{0} '.format(MULTI_EMAIL_FIELD_VALUE_SPLITTER).join(
                self.data.to_email
            )
        else:
            # Assume that it's string
            to_email = self.data.to_email

        context = {
            'to_name': safe_text(self.data.to_name),
            'to_email': to_email,
            'subject': safe_text(self.data.subject),
        }
        return render_to_string('mail/plugin_data_repr.html', context)


form_wizard_handler_plugin_registry.register(MailWizardHandlerPlugin)



from __future__ import absolute_import

from django import forms
from django.utils.translation import ugettext_lazy as _

from fobi.base import BaseFormFieldPluginForm, get_theme

__title__ = 'fobi.contrib.plugins.form_elements.fields.datetime.forms'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('DateTimeInputForm',)

theme = get_theme(request=None, as_instance=True)


class DateTimeInputForm(forms.Form, BaseFormFieldPluginForm):
    """Form for ``DateTimeInputPlugin``."""

    plugin_data_fields = [
        ("label", ""),
        ("name", ""),
        ("help_text", ""),
        ("initial", ""),
        ("input_formats", ""),
        ("required", False)
    ]

    label = forms.CharField(
        label=_("Label"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    name = forms.CharField(
        label=_("Name"),
        required=True,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    help_text = forms.CharField(
        label=_("Help text"),
        required=False,
        widget=forms.widgets.Textarea(
            attrs={'class': theme.form_element_html_class}
        )
    )
    initial = forms.DateTimeField(
        label=_("Initial"),
        required=False,
        widget=forms.widgets.DateTimeInput(
            attrs={'class': theme.form_element_html_class, 'type': 'datetime'}
        )
    )
    input_formats = forms.CharField(
        label=_("Input formats"),
        required=False,
        widget=forms.widgets.TextInput(
            attrs={'class': theme.form_element_html_class}
        )
    )
    required = forms.BooleanField(
        label=_("Required"),
        required=False,
        widget=forms.widgets.CheckboxInput(
            attrs={'class': theme.form_element_checkbox_html_class}
        )
    )

    def clean_initial(self):
        """Clean the initial value."""
        initial = self.cleaned_data['initial']
        try:
            return initial.strftime("%Y-%m-%d %H:%M:%S")
        except Exception as err:
            return initial



from django.utils.translation import ugettext_lazy as _

from fobi.base import BaseTheme, theme_registry

from . import UID

__title__ = 'fobi.contrib.themes.foundation5.fobi_themes'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Foundation5Theme',)


class Foundation5Theme(BaseTheme):
    """Foundation5 theme.

    Based on the "Workspace" example of the Foundation 5.
    Click `here <http://foundation.zurb.com/templates/contact.html>`_ for
    more.
    """
    uid = UID
    name = _("Foundation 5")

    media_css = (
        'foundation5/css/foundation.min.css',
        'foundation5/css/foundation_fobi_extras.css',
        'foundation5/icons/3/icons/foundation-icons.css',
        # 'foundation5/icons/2/accessibility/stylesheets/'
        # 'accessibility_foundicons.css',
        # 'foundation5/css/foundation_template-4.css', # Foundation Template 4
        # 'foundation5/css/normalize.css',
        # 'css/fobi.core.css',
    )

    media_js = (
        'foundation5/js/vendor/modernizr.js',
        # 'foundation5/js/foundation.min.js',
        # 'js/jquery-1.10.2.min.js',
        'foundation5/js/vendor/jquery.js',
        # 'foundation5/js/vendor/fastclick.js',
        'jquery-ui/js/jquery-ui-1.10.4.custom.min.js',
        # 'foundation5/js/foundation_template-4.js', # Foundation Template 4
        'foundation5/js/foundation.min.js',
        'js/fobi.core.js',
        'js/jquery.slugify.js',
        'foundation5/js/foundation5_fobi_extras.js',  # Theme specific scripts
    )

    # footer_text = '&copy; django-fobi example site 2014'

    # *************************************************************************
    # ********************** Form HTML specific *******************************
    # *************************************************************************
    form_element_html_class = 'form-control'
    form_element_checkbox_html_class = 'checkbox'

    # Important
    form_view_form_entry_option_class = 'fi-list'

    # Important
    form_edit_form_entry_option_class = 'fi-page-edit'

    # Important
    form_delete_form_entry_option_class = 'fi-page-delete'

    # Important
    form_list_container_class = 'inline-list'

    # *************************************************************************
    # ********************** Templates specific *******************************
    # *************************************************************************
    master_base_template = 'foundation5/_base.html'
    base_template = 'foundation5/base.html'

    form_ajax = 'foundation5/snippets/form_ajax.html'
    form_snippet_template_name = 'foundation5/snippets/form_snippet.html'
    form_properties_snippet_template_name = \
        'foundation5/snippets/form_properties_snippet.html'
    messages_snippet_template_name = \
        'foundation5/snippets/messages_snippet.html'
    form_non_field_and_hidden_errors_snippet_template = \
        'foundation5/snippets/form_non_field_and_hidden_errors_snippet.html'

    add_form_element_entry_template = \
        'foundation5/add_form_element_entry.html'
    add_form_element_entry_ajax_template = \
        'foundation5/add_form_element_entry_ajax.html'

    add_form_handler_entry_template = \
        'foundation5/add_form_handler_entry.html'
    add_form_handler_entry_ajax_template = \
        'foundation5/add_form_handler_entry_ajax.html'

    create_form_entry_template = 'foundation5/create_form_entry.html'
    create_form_entry_ajax_template = \
        'foundation5/create_form_entry_ajax.html'

    dashboard_template = 'foundation5/dashboard.html'
    forms_list_template = 'foundation5/forms_list.html'

    edit_form_element_entry_template = \
        'foundation5/edit_form_element_entry.html'
    edit_form_element_entry_ajax_template = \
        'foundation5/edit_form_element_entry_ajax.html'

    edit_form_entry_template = 'foundation5/edit_form_entry.html'
    edit_form_entry_ajax_template = 'foundation5/edit_form_entry_ajax.html'

    edit_form_handler_entry_template = \
        'foundation5/edit_form_handler_entry.html'
    edit_form_handler_entry_ajax_template = \
        'foundation5/edit_form_handler_entry_ajax.html'

    form_entry_submitted_template = 'foundation5/form_entry_submitted.html'
    form_entry_submitted_ajax_template = \
        'foundation5/form_entry_submitted_ajax.html'

    view_form_entry_template = 'foundation5/view_form_entry.html'
    view_form_entry_ajax_template = 'foundation5/view_form_entry_ajax.html'

    import_form_entry_template = 'foundation5/import_form_entry.html'
    import_form_entry_ajax_template = \
        'foundation5/import_form_entry_ajax.html'


theme_registry.register(Foundation5Theme)



import datetime

from fobi.base import get_theme
from fobi.form_importers import get_form_impoter_plugin_urls
from fobi.helpers import StrippedRequest

__title__ = 'fobi.context_processors'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'dynamic_values',
    'form_importers',
    'theme',
)


def theme(request):
    """Get active theme.

    :param django.http.HttpRequest request:
    :return fobi.base.BaseTheme: Instance of ``fobi.base.BaseTheme``.
    """
    return {'fobi_theme': get_theme(request, as_instance=True)}


def dynamic_values(request):
    """Dynamic values exposed to public forms."""
    return {
        'fobi_dynamic_values': {
            'request': StrippedRequest(request),
            'now': datetime.datetime.now(),
            'today': datetime.date.today(),
        }
    }


def form_importers(request):
    """Form importers."""
    return {
        'form_importers': get_form_impoter_plugin_urls(),
    }
